'use strict';

angular.module('sdbgui')
    .component('databases', {
        templateUrl: '/static/components/sdbgui-databases/sdbgui-databases.template.html',
        controller: [
            '$q', '$scope', '$location', '$interval', 'sortingService', 'slashDB',
            function QueryController($q, $scope, $location, $interval, sortingService, slashdb) {
                var ctrl = this;
                ctrl.sort = sortingService;
                ctrl.connectStatusesInterval = null;
                ctrl.dbList = [];
                ctrl.Statuses = {
                    Failed: 'Failed',
                    Connected: 'Connected',
                    Disconnected: 'Disconnected',
                    all: []
                };
                ctrl.Statuses.all = [ctrl.Statuses.Failed, ctrl.Statuses.Connected, ctrl.Statuses.Disconnected];

                function init() {
                    $scope.searchable_fields = ['db_id', 'db_type', 'desc'];

                    ctrl.dbDict = {};
                    ctrl.dbList = [];
                    ctrl.settings=slashdb.settings;
                    $scope.updateList();
                    getConnectStatuses();

                    $scope.$on('$destroy', function () {
                        $interval.cancel(ctrl.connectStatusesInterval);
                    });
                }

                function getConnectStatuses(){
                    if (ctrl.dbList.len == 0){
                        return null
                    }

                    slashdb.get('/settings/reflect-status.json', {}, false, false)
                        .then(function(response){
                            var connectStatuses = response.data;
                            var stopInterval = true;
                            angular.forEach(ctrl.dbList, function (dbconf) {
                                dbconf.connect_status = connectStatuses[dbconf.db_id].model;
                                stopInterval = stopInterval && dbconf.connect_status != 'Connecting'
                            });
                            if (stopInterval) cancelConnectStatusInterval();
                        })
                        .catch(function(){
                            cancelConnectStatusInterval();
                        })
                }

                function updateConnectStatuses(){
                    if (ctrl.connectStatusesInterval == null){
                        ctrl.connectStatusesInterval = $interval(getConnectStatuses, 1000, 0);
                    }
                }

                function cancelConnectStatusInterval(){
                    $interval.cancel(ctrl.connectStatusesInterval);
                    ctrl.connectStatusesInterval = null;
                }

                $scope.updateList = function() {
                    slashdb.get('/db.json', {params: {'guidata': ''}}, false, false)
                        .then(function (response) {
                            var dbList = [],
                                dbDict = {},
                                dataKeys = Object.keys(response.data),
                                refid_prefix = slashdb.settings.refid_prefix,
                                db, dataKey;

                            for (var idx = 0, dataKeysL = dataKeys.length; idx < dataKeysL; idx++) {
                                dataKey = dataKeys[idx];
                                if (dataKey !== refid_prefix) {
                                    db = angular.extend({
                                        db_id: dataKey
                                    }, response.data[dataKey]);
                                    dbList.push(db);
                                    dbDict[dataKey] = db;
                                }
                            }

                            ctrl.listAlert = '';
                            // update ctrl stuff only once
                            ctrl.dbList = dbList;
                            ctrl.dbDict = dbDict;

                            // refresh slashDB.settings
                            slashdb.setupSettings();
                            updateConnectStatuses();
                        })
                        .catch(function (response) {
                            ctrl.listAlert = response.headers('Warning');
                            ctrl.dbList = [];
                        });
                };

                $scope.toggleDBState = function (dbConfig) {
                    var connectStatus = dbConfig.connect_status;
                    dbConfig.connect_status = '';
                    getConnectStatuses();
                    slashdb.toggleDB(dbConfig.db_id, connectStatus).finally(function () {
                        $scope.updateList();
                    })
                };

                $scope.getStatusStyle = function (status) {
                    if (status == ctrl.Statuses.Connected) return 'bg-success';
                    if (status == ctrl.Statuses.Failed) return 'bg-danger';
                };

                $scope.isLoading = function (dbConfig) {
                    return ctrl.Statuses.all.indexOf(dbConfig.connect_status) < 0;
                };

                init();
            }
        ]
    });
