// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
    .controller('DBConfigMessageBoxCtrl', ['$window', '$timeout', '$q',
        '$rootScope', '$scope', '$uibModalInstance', 'slashDB', 'configId',
        function ($window, $timeout, $q, $rootScope, $scope, $uibModalInstance,
                  slashdb, configId) {
            var ctrl = this;
            $scope.dbId = configId;

            function init(){
                getReflectStatus($scope.dbId);
                getDbConfig($scope.dbId);
            }

            function getDbConfig(dbId) {
                slashdb.get(
                    '/dbdef/' + dbId + '.json',
                    {params: {guidata: ''}},
                    false, false)
                    .then((response) => {
                        let config = response.data;
                        $scope.editable = config.editable;
                        $scope.viewable = config.viewable;
                        $scope.executable = config.executable;
                    })
                    .catch((response) => {
                        if (response.status === 403) {
                            $scope.editable = false;
                            $scope.viewable = false;
                            // If db_id is obtained, then it should be executable.
                            $scope.executable = true;
                        } else {
                            console.log(response);
                        }
                    })
            }

            function getReflectStatus(dbId){
                slashdb.get('/settings/reflect-status/' + dbId + '.json', null, false, false)
                .then(function(response){
                        $scope.reflectErrorMessage = response.data.message;
                        $scope.connectStatus = response.data.model;
                    }
                )
            }

            $scope.retry = function(){
                var eventData = {
                    configId: configId,
                    doReflect: true
                };
                $uibModalInstance.close();
                $rootScope.$broadcast('open-db-wizard', eventData);
            };

            $scope.modify = function(){
                var eventData = {
                    configId: configId,
                    doReflect: undefined
                };
                $uibModalInstance.close();
                $rootScope.$broadcast('open-db-editor', eventData);
            };

            $scope.close = function(){
                $uibModalInstance.dismiss();
            };

            init();
        }
    ]);