// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
        .controller('DBConfigModalCtrl', ['$window', '$timeout', '$q',
            '$rootScope', '$scope', '$uibModalInstance', 'slashDB', 'configId',
            'selectizeService', 'looseAccess',
        function ($window, $timeout, $q, $rootScope, $scope, $uibModalInstance,
                  slashdb, configId, selectizeService, looseAccess) {
            const ctrl = this;
            ctrl.sqlalchemyFormFields = {
                mssql: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                oracle : ['db_host', 'db_port', 'user', 'password', 'sid', 'db_schema'],
                db2: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                mysql: ['db_host', 'db_port', 'user', 'password', 'db_name'],
                mariadb: ['db_host', 'db_port', 'user', 'password', 'db_name'],
                postgresql: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                sqlite: ['file_path'],
                sqlcipher: ['file_path', 'password'],
            };
            // to add non sqlachemy form fields create new object and put it in
            // call at the next line
            $scope.formFields = Object.assign({}, ctrl.sqlalchemyFormFields);
            ctrl.mysqlLikeDb = ['mysql', 'mariadb'];
            ctrl.fileBasedDB = ['sqlite', 'sqlcipher'];
            $scope.newDbConfig = !Boolean(configId);
            $scope.configId = configId;
            $scope.deleteVisible = !$scope.newDbConfig;
            $scope.username = slashdb.settings.user;
            // store connstr when checkbox unchecked. stored until page reload
            $scope.tempConnstr = null;
            var allCharsets = [];
            var mysqlCharsets = [];

            slashdb.settings.py_encodings.forEach(function(charset){
                allCharsets.push(charset.label);
                if (charset.mysql) mysqlCharsets.push(charset.label)
            });

            $scope.required_fields = {};

            $scope.getCharsets = function(dbType){
                if (ctrl.mysqlLikeDb.includes(dbType)) {
                    return mysqlCharsets;
                } else {
                    return allCharsets;
                }
            };

            var init = function () {
                $scope.selectizeService = selectizeService;
                selectizeService.updateUserOptions(null);
                $scope.okVisible = false;
                $scope.modalAlert = '';
                $scope.savingInProgress = false;
                resetDbConfig();
                getDbConfig();
            };

            /** @returns {Promise} */
            var getDbConfig = function () {
                if ($scope.newDbConfig) {
                    return $q(function (resolve, reject) {
                        resolve();
                    });
                }

                return $q(function (resolve, reject) {
                    slashdb.get('/dbdef/' + $scope.configId + '.json', {
                            params: {
                                guidata: ''
                            }
                        }, false, false)
                        .then(function (response) {
                            $scope.configData = response.data;
                            $scope.deleteVisible = $scope.configData.editable;
                            $scope.configData.override = Boolean($scope.configData.connection);
                            selectizeService.updateUserOptions($scope.configData);
                            var dbEncoding = $scope.configData.db_encoding;
                            // preserve original encoding in dbconfig
                            if (!allCharsets.includes(dbEncoding)) allCharsets.push(dbEncoding);
                            if (ctrl.mysqlLikeDb.includes($scope.configData.db_type)) {
                                if (!mysqlCharsets.includes(dbEncoding)) mysqlCharsets.push(dbEncoding);
                            }
                            // refresh slashDB.settings
                            slashdb.setupSettings();
                            resolve();
                        });
                });
            };

            var resetDbConfig = function () {
                // stores database definition model, some initial values
                $scope.configData = {
                    db_type: 'sqlite',
                    autoload: false,
                    autoconnect: true,
                    editable: true,
                    autoload_user: {dbuser: null, dbpass: null},
                };
            };

            /** @returns {boolean} */
            $scope.requiresCredentials = function () {
                return $scope.configData.use_user_db_credentials_for_mapping &&
                    $scope.configData.db_type !== 'sqlite' &&
                    $scope.configData.autoload &&
                    $scope.isSQLAlchemy();
            };

            $scope.isSQLAlchemy = function () {
                return ctrl.sqlalchemyFormFields.hasOwnProperty($scope.configData.db_type);
            };

            $scope.delete = function () {
                slashdb.deleteDBDef($scope.configId)
                    .then(function (response) {
                        $uibModalInstance.dismiss();

                        // refresh slashDB.settings
                        slashdb.setupSettings();
                    })
                    .catch(function (response) {
                        $scope.modalAlert = response.headers('Warning');
                    });
            };

            /**
             * @param {string} dbId
             * @param {boolean} useForMapping
             */
            var sendAddBtnMsg = function (dbId, useForMapping) {
                var addBtnMsg = useForMapping ? '' :
                    'Click to associate this user with database "' + dbId + '" login / password.';
                $rootScope.$broadcast('open-user-settings-modal', addBtnMsg);
            };

            $scope.fieldVisible = function(fieldName) {
                return $scope.formFields[$scope.configData.db_type].includes(fieldName);
            };

            var setInvalidFields = function(errors) {
                var invalid = [];
                // get list of invalid fields
                errors.forEach(
                    function (err, _, __) {
                        if (err['type'] === 'INVALID') {
                            invalid.push(err['field']);
                        }
                    }
                );

                // mark fields on form as invalid
                invalid.forEach(function (value) {
                    // check that field exist on form
                    if (!$scope.configForm.hasOwnProperty(value)) {
                        console.log('Warning - bad property ' + value);
                        return
                    }
                    // set invalid
                    $scope.configForm[value].$setValidity('server', false)
                });
                return invalid
            };

            var setRequiredFields = function (errors) {
                const alwaysRequired = ['db_id', 'db_type', 'db_encoding'];

                let required = {};
                errors.forEach(
                    function (err, _, __) {
                        if (err['type'] === 'MISSING') {
                            required[err['field']] = true;
                        }
                    }
                );

                alwaysRequired.forEach(function (value) {
                    required[value] = true;
                });

                $scope.required_fields = required;
            };

            var getUserFriendlyErrorMessages = function (errors) {
                return 'Errors found in incoming database definition: ' +
                    errors.map(
                    function (err, _, __) {
                        if (err['type'] === 'MISSING') {
                            return err['description']
                        }

                        if (err['type'] === 'INVALID') {
                            if (err['field'] === 'db_encoding') {
                                return 'Database charset is obligatory'
                            }
                            return  err['description']
                        }

                        if (err['type'] === 'EXTRA') {
                            return err['description'] + " - try to reload the page"
                        }

                        // else
                        return 'Unexpected error - try to reload the page'
                    }
                ).join(', ') + '.'
            };

            var resetErrors = function() {
                $scope.modalAlert = '';
                $scope.required_fields = {};
                for (var key in $scope.configForm) {
                    var value = $scope.configForm[key];
                    if (typeof value === 'object'
                        && value.hasOwnProperty('$modelValue')) {
                        value.$setValidity('server', true)
                    }
                }
            };
            $scope.getDropdownDbTypes = () => {
                // return only licensed db types and the current one
                const sdbDbTypes = slashdb.settings.dbtypes;
                let dropdownItems = {};
                for (const db_type in sdbDbTypes) {
                    if (sdbDbTypes[db_type]['licensed']
                        || db_type === $scope.configData.db_type) {
                        dropdownItems[db_type] = sdbDbTypes[db_type]['userfriendly_name'];
                    }
                }
                return dropdownItems;
            };

            $scope.updateConnstr = function() {
                if ($scope.configData.override) {
                    $scope.configData.connection = $scope.tempConnstr;
                } else {
                    $scope.tempConnstr = $scope.configData.connection;
                    $scope.configData.connection = null;
                }
            };

            $scope.resetError = function (field) {
                $scope.configForm[field].$setValidity('server', true)
            };

            var setErrors = function(errors, description) {
                if (errors !== undefined && errors.length > 0){
                    $scope.modalAlert = getUserFriendlyErrorMessages(errors);

                } else {
                    $scope.modalAlert = description;
                }
                setInvalidFields(errors);
                setRequiredFields(errors);
            };

            var stripReadOnlyItems = function(rawData) {
                var payload = Object.assign({}, rawData);
                delete payload.editable;
                delete payload.viewable;
                delete payload.executable;
                delete payload.connect_status;
                delete payload.override;

                return payload
            };

            let nullifyNonSqliteFields = (payload) => {
                payload.db_port = null;
                payload.db_host = null;
                payload.db_schema = null;

                return payload
            };

            $scope.save = function () {
                $scope.savingInProgress = true;
                let payload = stripReadOnlyItems($scope.configData);

                if (ctrl.fileBasedDB.includes($scope.configData.db_type)) {
                    // remove non relevant fields
                    payload = nullifyNonSqliteFields(payload);
                } else {
                    if (!$scope.configForm.db_port.$error.pattern) {
                        payload.db_port = parseInt(payload.db_port, 10);
                    }
                    if (ctrl.mysqlLikeDb.includes($scope.configData.db_type)) {
                        // mysql do not have schema
                        payload.db_schema = null;
                    }
                }

                resetErrors();

                if ($scope.newDbConfig) {
                    slashdb.createDBDef(payload)
                        .then(function (response) {
                            /** @type {boolean} */
                            var useForMapping = $scope.configData.use_user_db_credentials_for_mapping || false;

                            blinkOK();
                            $scope.newDbConfig = false;
                            $scope.configId = $scope.configData.db_id;
                            getDbConfig().then(function () {
                                sendAddBtnMsg($scope.configId, useForMapping);
                            });
                            setRequiredFields([]);
                        })
                        .catch(function (response) {
                            setErrors(response.data.errors, response.data.description);
                        }).finally((response) => {
                            $scope.savingInProgress = false
                        });
                } else {
                    slashdb.updateDBDef($scope.configData.db_id, payload)
                        .then(function (response) {
                            blinkOK();
                            getDbConfig();
                            setRequiredFields([]);
                        })
                        .catch(function (response) {
                            setErrors(response.data.errors, response.data.description);
                        }).finally((response) => {
                            $scope.savingInProgress = false
                        });
                }
            };

            $scope.close = function () {
                $uibModalInstance.dismiss();
            };

            $scope.hasAccess = looseAccess.hasAccess;

            var blinkOK = function () {
                $scope.okVisible = true;
                $timeout(function () {
                    $scope.okVisible = false;
                }, 1000);
            };
            init();
        }
    ])
    .directive('emptyToNull', function () {
    return {
        restrict: 'A',
        require: 'ngModel',
        link: function (scope, elem, attrs, ctrl) {
            ctrl.$parsers.push(function(viewValue) {
                if(viewValue === "") {
                    return null;
                }
                return viewValue;
            });
        }
    };
});
