// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
    .controller('DBConfigWizardCtrl', ['$scope', '$uibModalInstance',
        'wizardOptions', '$rootScope', '$http', '$q', '$location', '$timeout',
        '$interval', '$window', 'slashDB',
        function($scope, $uibModalInstance, wizardOptions, $rootScope, $http,
                 $q, $location, $timeout, $interval, $window, slashdb){
            var ctrl = this;
            var _testConnection = {showSpinner: false, success: '', error: ''};
            var _serverInfoKeys = [
                'db_name', 'db_type', 'file_path', 'db_host', 'db_port',
                'db_schema', 'password', 'user',
            ];
            var _emptyDict = {};
            var reflectStatusInterval = null;
            var reflectAutoCancel = 3;
            $scope.settings = slashdb.settings;

            function init (){
                $scope.username = slashdb.settings.user;
                $scope.activeTab = 0;
                $scope.tabOptions = [
                    {showBack: false, showConnect: false, showNext: false, disableNext: false, disableTest:false, showClose: false},
                    {showBack: true, showConnect: false, showNext: true, disableNext: false, disableTest:false, showClose: false},
                    {showBack: true, showConnect: false, showNext: true, disableNext: false, disableTest:false, showClose: false},
                    {showBack: false, showConnect: false, showNext: false, disableNext: false, disableTest:false, showClose: true}
                ];
                $scope.formFields = {
                    mssql: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                    oracle : ['db_host', 'db_port', 'user', 'password', 'sid', 'db_schema'],
                    db2: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                    mysql: ['db_host', 'db_port', 'user', 'password', 'db_name'],
                    mariadb: ['db_host', 'db_port', 'user', 'password', 'db_name'],
                    postgresql: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                    sqlite: ['file_path'],
                    sqlcipher: ['file_path', 'password'],
                };
                ctrl.mysqlLikeDb = ['mysql', 'mariadb'];
                ctrl.fileBasedDB = ['sqlite', 'sqlcipher'];

                resetModels();

                // Wizard opened just to show status.
                if (wizardOptions !== undefined) {
                    showOnlyStatus(wizardOptions.configId, wizardOptions.doReflect);
                }

                $scope.renameReflectStatus = function(status){
                    var stMap = {
                        0: 'Listed',
                        1: 'In progress',
                        2: 'OK',
                        3: 'Skipped.'
                    };

                    return stMap[status];
                };

                $scope.renameReflectError = function(error){
                    if (error === 'no pk'){
                        return 'No primary key.'
                    } else {
                        return error
                    }
                };

                var allCharsets = [];
                var mysqlCharsets = [];

                slashdb.settings.py_encodings.forEach(function(charset){
                    allCharsets.push(charset.label);
                    if (charset.mysql) mysqlCharsets.push(charset.label)
                });

                $scope.required_fields = {};

                $scope.getCharsets = function(dbType){
                    if (ctrl.mysqlLikeDb.includes(dbType)) {
                        return mysqlCharsets;
                    } else {
                        return allCharsets;
                    }
                };

                $scope.$on('$destroy', function() {
                    cancelReflectStatus();
                });
            }

            function showOnlyStatus(openedConfigId, doReflect){
                // open status when openedConfigId passed
                $scope.apiInfo.db_id = openedConfigId;
                $scope.activeTab = 3;
                if (doReflect){
                    $scope.reflect(openedConfigId)
                } else {
                    $scope.getReflectStatus(openedConfigId, true);
                }
            }

            function resetModels(){
                $scope.apiFormErrors = angular.copy(_emptyDict);
                $scope.serverInfo = angular.copy(_emptyDict);
                $scope.serverInfo.db_encoding = 'utf-8';
                $scope.testConnection = angular.copy(_testConnection);
                $scope.apiInfo = {
                    newConfig: true
                };
            }

            function cancelReflectStatus(){
                if(reflectStatusInterval){
                    $interval.cancel(reflectStatusInterval)
                }
            }

            function saveConfig () {
                var configData = {
                    "db_id": $scope.apiInfo.db_id,
                    "db_type": $scope.serverInfo.db_type,
                    "db_host": $scope.serverInfo.db_host,
                    "db_port": parseInt($scope.serverInfo.db_port, 10),
                    "db_name": ctrl.fileBasedDB.includes($scope.serverInfo.db_type) ?
                        $scope.serverInfo.file_path : $scope.serverInfo.db_name,
                    // modify this to add connection override to wizard
                    "connection": null,
                    "db_encoding": $scope.serverInfo.db_encoding,
                    "autoload_user": {
                        "dbuser": $scope.serverInfo.user,
                        "dbpass": $scope.serverInfo.password
                    },
                    "autoconnect": Boolean($scope.apiInfo.autoconnect),
                    "autoload": Boolean($scope.apiInfo.autoload),
                    "read": [],
                    "write": [],
                    "execute": [],
                    "foreign_keys": {},
                    "db_schema": $scope.serverInfo.db_schema,
                    "alternate_key": {},
                    "excluded_columns": {},
                    "desc": $scope.apiInfo.desc,
                    "use_user_db_credentials_for_mapping": $scope.apiInfo.useCredentials || false
                };
                if ($scope.apiInfo.newConfig){
                    return slashdb.createDBDef(configData)
                    .then(function (response) {
                        $scope.apiInfo.newConfig = false;
                        return 'Database endpoint ' + configData.db_id + ' saved successfully.'
                    })
                    .catch(function (response) {
                        $scope.apiInfo.newConfig = true;
                        return $q.reject(response.data);
                    });
                } else {
                    return slashdb.updateDBDef(configData.db_id, configData)
                    .then(function (response) {
                        return 'Database endpoint ' + configData.db_id + ' saved successfully.'
                    })
                    .catch(function (response) {
                        return $q.reject(response.data);
                    });
                }

            }

            $scope.fieldVisible = function(fieldName) {
                return $scope.formFields[$scope.serverInfo.db_type].includes(fieldName)
            };

            $scope.selectDatabase = function(dbType, isLicensed){
                if (!isLicensed) {
                    // open in new tab
                    $window.open('license', '_blank');
                    return
                }

                resetModels();
                $scope.serverInfo.db_type = dbType;
                $scope.activeTab++;
            };

            $scope.fixServerInfo = function(){
                for (var i=0; i < _serverInfoKeys.length; i++){
                    var key = _serverInfoKeys[i];
                    if ($scope.serverInfo[key] === undefined){
                        $scope.serverInfo[key] = null;
                    }
                }
            };

            $scope.testDbConnection = function(){
                $scope.fixServerInfo();
                $scope.testConnection = angular.copy(_testConnection);
                $scope.testConnection.showSpinner = true;
                $scope.tabOptions[$scope.activeTab].disableNext = true;
                $scope.tabOptions[$scope.activeTab].disableTest = true;
                $scope.validateServerForm = true;
                return $http.post('/settings/check-database-connection.json', $scope.serverInfo)
                    .then(function successCallback(response){
                        $scope.testConnection.success = response.data;
                    })
                    .catch(function errorCallback(response){
                        $scope.testConnection.error = response.data.description;
                        return $q.reject();
                    })
                    .finally(function(){
                        $scope.testConnection.showSpinner = false;
                        $scope.tabOptions[$scope.activeTab].disableNext = false;
                        $scope.tabOptions[$scope.activeTab].disableTest = false;
                    });
            };

            function checkReflectionStatus(db_id){
                slashdb.get('/settings/reflect-status/' + db_id + '.json', null, false, false)
                .then(function(response){

                    if (response.data.message){
                        if (wizardOptions !== undefined && wizardOptions.configId) {
                            $scope.close();
                            $rootScope.$broadcast('open-messagebox', db_id);
                        } else {
                            $scope.reflectErrorMessage = response.data.message;
                        }
                    }

                    $scope.reflectStatus = response.data.tables;
                    if ($scope.reflectStatus.length === 0){
                        $scope.reflectProgress = 1;
                        $scope.reflectProgressMax = 50;
                    } else {
                        $scope.reflectProgressMax = $scope.reflectStatus.length + 2;
                        $scope.reflectProgress = 1 + $scope.reflectStatus.filter(function(st){return st.status > 0}).length;
                    }
                    // first check status reflectAutoCancel number of time
                    // then cancel interval if connect status is not connecting
                    $scope.connectStatus = response.data.model;
                    if (reflectAutoCancel < 0 && $scope.connectStatus !== 'Connecting') {
                        $scope.reflectProgress++;
                        cancelReflectStatus();
                    }
                    reflectAutoCancel--;
                });
            }

            $scope.reflect = function(db_id){
                $scope.reflectProgress = 1;
                $scope.reflectProgressMax = 50;
                $scope.connectStatus = 'Connecting';
                reflectAutoCancel = 3;
                $scope.reflectErrorMessage = '';
                var reflectPromise = slashdb.dbOn(db_id)
                .then(function () {})
                .catch(function (response) {
                    // only if not Request or Gateway Timeout
                    if (![408, 504].includes(response.status)) {
                        $scope.reflectErrorMessage = response.data.description;
                        console.log($scope.reflectErrorMessage);
                    }
                    return $q.reject();
                });
                $scope.getReflectStatus(db_id, false);
                return reflectPromise;
            };

            $scope.getReflectStatus = function(db_id, imidiately){
                if (imidiately){
                    checkReflectionStatus(db_id, false);
                }
                reflectStatusInterval = $interval(checkReflectionStatus, 1000, 0, false, db_id, true);
            };

            $scope.connect = function(){
                $scope.visibleConnectionNotEstablished = false;
                $scope.testDbConnection()
                .then(function (response) {
                    $scope.reflect($scope.apiInfo.db_id);
                })
                .catch(function (response) {
                    $scope.visibleConnectionNotEstablished = true;
                });
            };

            $scope.visibleReflectTable = function(){
                var showTableWhen = ['Connected', 'Connecting'];
                return $scope.connectStatus && showTableWhen.includes($scope.connectStatus)
            };

            $scope.updateEndpointUrl = function(){
                var port = $location.port(),
                    protocol = $location.protocol(),
                    host = $location.host(),
                    url = '';

                if (protocol === 'http' && port === '80' || protocol === 'https' && port === '443'){
                    url = protocol + '://' + host + '/db/'
                } else {
                    url = protocol + '://' + host + ':' + port + '/db/'
                }

                url += $scope.apiInfo.db_id;

                $scope.apiInfo.endpointUrl = url
            };

            $scope.back = function(){
                $scope.activeTab--;
            };

            $scope.next = function(){
                if ($scope.activeTab == 1) {
                    $scope.validateServerForm = true;
                    // wait for next digest
                    $timeout($scope.goTo2);
                } else if ($scope.activeTab == 2) {
                    $scope.goTo3();
                } else {
                    $scope.activeTab++;
                }
            };

            $scope.goTo1 = function(){
                $scope.activeTab = 1;
            };

            $scope.goTo2 = function(){
                if (ctrl.serverForm.$valid) {
                    // Database Endpoint Id proposal.
                    if (!$scope.apiInfo.db_id) {
                        var dbId;
                        if (ctrl.fileBasedDB.includes($scope.serverInfo.db_type)) {
                            dbId = $scope.serverInfo.file_path;
                            // Take last file path segment and remove extension.
                            dbId = dbId.split('/').pop().replace(/\.[^/.]+$/, "")
                        } else {
                            dbId = $scope.serverInfo.db_name;
                        }
                        // Replace illegal characters with dash. Trim dashes.
                        dbId = dbId.replace(/[^a-zA-Z0-9@_~-]/g, "-");
                        dbId = dbId.replace(/^-+|-+$/g, "");
                        $scope.apiInfo.db_id = dbId;
                    }
                    $scope.updateEndpointUrl();
                    $scope.activeTab = 2;
                }
            };

            $scope.goTo3 = function(){
                $scope.apiFormErrors = {};
                saveConfig()
                .then(function(message){
                    $scope.connect();
                    $scope.apiFormErrors = {};
                    $scope.activeTab = 3;
                })
                .catch(function(reject){
                    $scope.apiFormErrors = reject.errors;
                })
            };

            $scope.close = function() {
                $uibModalInstance.dismiss();
            };

            init();
        }
    ])
    .directive('emptyToNull', function () {
        return {
            restrict: 'A',
            require: 'ngModel',
            link: function (scope, elem, attrs, ctrl) {
                ctrl.$parsers.push(function(viewValue) {
                    if(viewValue === "") {
                        return null;
                    }
                    return viewValue;
                });
            }
        };
    });
