// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
    .component('dbconfigs', {
        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs.template.html',
        controller: ['$uibModal', '$scope', '$interval', 'sortingService', 'slashDB',
            function DbConfigController($uibModal, $scope, $interval, sortingService, slashdb) {
                var ctrl = this;

                function init() {
                    ctrl.sort = sortingService;
                    ctrl.connectStatus = {};
                    ctrl.addConfigVisible = false;
                    ctrl.dbconfigs = [];
                    ctrl.connectStatusesInterval = null;
                    ctrl.settings = slashdb.settings;
                    $scope.searchable_fields = ['db_id', 'db_type', 'desc'];

                    ctrl.updateConfigList();
                    getConnectStatuses();

                    $scope.$on('$destroy', function () {
                        $interval.cancel(ctrl.connectStatusesInterval);
                    });

                    $scope.$on('open-messagebox', function(event, configId){
                        ctrl.openMessageBox(configId);
                    });

                    $scope.$on('open-db-wizard', function(event, eventData){
                        ctrl.openWizard(eventData);
                    });

                    $scope.$on('open-db-editor', function(event, eventData){
                        ctrl.openModal(eventData.configId);
                    });

                    // open wizard on ?new in qs
                    if (window.location.search.includes('new')) {
                        ctrl.openWizard(undefined);
                    }
                }

                function getConnectStatuses(){
                    if (ctrl.dbconfigs.len == 0){
                        return null
                    }

                    slashdb.get('/settings/reflect-status.json', {}, false, false)
                        .then(function(response){
                            var connectStatuses = response.data;

                            var stopInterval = true;
                            angular.forEach(ctrl.dbconfigs, function (dbconf) {
                                dbconf.connect_status = connectStatuses[dbconf.db_id].model;
                                stopInterval = stopInterval && dbconf.connect_status != 'Connecting'
                            });
                            if (stopInterval) cancelConnectStatusInterval();
                        })
                        .catch(function(){
                            cancelConnectStatusInterval();
                        })
                }

                function updateConnectStatuses(){
                    if (ctrl.connectStatusesInterval == null){
                        ctrl.connectStatusesInterval = $interval(getConnectStatuses, 1000, 0);
                    }
                }

                function cancelConnectStatusInterval(){
                    $interval.cancel(ctrl.connectStatusesInterval);
                    ctrl.connectStatusesInterval = null;
                }

                ctrl.updateConfigList = function () {
                    slashdb.get('/dbdef.json', {
                        params: {
                            guidata: ''
                        }
                    }, false, false)
                        .then(function (response) {
                            var dbconfigs = [];
                            ctrl.addConfigVisible = response.data.create_database;
                            // @ts-ignore
                            angular.forEach(response.data.dbdefs, function (dbconf) {
                                dbconfigs.push(dbconf);
                            });
                            ctrl.listAlert = '';
                            ctrl.dbconfigs = dbconfigs;
                            updateConnectStatuses();
                        })
                        .catch(function (response) {
                            ctrl.listAlert = response.headers('Warning');
                            ctrl.dbconfigs = [];
                        });
                };

                ctrl.Statuses = {
                    Failed: 'Failed',
                    Connected: 'Connected',
                    Disconnected: 'Disconnected',
                    all: []
                };
                ctrl.Statuses.all = [ctrl.Statuses.Failed, ctrl.Statuses.Connected, ctrl.Statuses.Disconnected];

                ctrl.getStatusStyle = function (status) {
                    if (status == ctrl.Statuses.Connected) return 'bg-success';
                    if (status == ctrl.Statuses.Failed) return 'bg-danger';
                };

                ctrl.isLoading = function (dbConfig) {
                    return !ctrl.Statuses.all.includes(dbConfig.connect_status);
                };

                ctrl.toggleDBState = function (dbConfig) {
                    var connectStatus = dbConfig.connect_status;
                    dbConfig.connect_status = '';
                    getConnectStatuses();
                    slashdb.toggleDB(dbConfig.db_id, connectStatus).finally(
                        function () {
                          ctrl.updateConfigList();
                    })
                };

                ctrl.openModal = function (configId) {
                    var modalInstance = $uibModal.open({
                        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs-modal.template.html',
                        size: 'lg',
                        backdrop: 'static',
                        controller: 'DBConfigModalCtrl',
                        resolve: {
                            configId: function () {
                                return configId;
                            }
                        }
                    });
                    modalInstance.result
                        .finally(function () {
                            // refresh config list after closing modal
                            ctrl.updateConfigList();
                        });
                };


                ctrl.openWizard = function(wizardOptions){
                    var modalInstance = $uibModal.open({
                        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs-wizard.template.html',
                        size: 'lg',
                        backdrop: 'static',
                        controller: 'DBConfigWizardCtrl',
                        controllerAs: 'ctrl',
                        resolve: {
                            wizardOptions: function () {
                                return wizardOptions;
                            }
                        }
                    });
                    modalInstance.result
                        .finally(function () {
                            // refresh config list after closing modal
                            ctrl.updateConfigList();
                        });
                };

                ctrl.openMessageBox = function(configId){
                    var modalInstance = $uibModal.open({
                        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs-messagebox.template.html',
                        size: 'lg',
                        windowTopClass: 'center-modal',
                        controller: 'DBConfigMessageBoxCtrl',
                        controllerAs: 'ctrl',
                        resolve: {
                            configId: function () {
                                return configId;
                            }
                        }
                    });
                    modalInstance.result
                        .finally(function (reason) {
                            // refresh config list after closing modal
                            ctrl.updateConfigList();
                        });
                };

                ctrl.openStatus = function(dbconfig){

                    if (dbconfig.connect_status == 'Connected' || dbconfig.connect_status == 'Connecting'){
                        ctrl.openWizard(
                            {
                                configId: dbconfig.db_id,
                                doReflect: false
                            })
                        ;
                    } else {
                        ctrl.openMessageBox(dbconfig.db_id);
                    }
                };

                init();
            }
        ]
    });
