angular.module('sdbgui', ['ngRoute', 'ngCookies', 'ngAnimate', 'ui.bootstrap', 'angularSlashDB', 'ngclipboard', 'ui.codemirror', 'selectize'])
.config(['$routeProvider', '$locationProvider',
        function config($routeProvider, $locationProvider) {
            $routeProvider
                .when('/', {
                    templateUrl: '/static/components/sdbgui-login/sdbgui-login-welcome.template.html',
                    controller: function(){
                        window.location = '/';
                    }
                })
                .when('/login', {
                    templateUrl: '/static/components/sdbgui-login/sdbgui-login-welcome.template.html',
                    controller: 'LoginController',
                    controllerAs: 'ctrl'
                })
                .when('/login/sso/:idp_id', {
                    templateUrl: '/static/components/sdbgui-login/sdbgui-login-in-progress.template.html',
                    controller: 'SSOController',
                    controllerAs: 'ctrl'
                })
                .when('/login/callback', {
                    templateUrl: '/static/components/sdbgui-login/sdbgui-login-in-progress.template.html',
                    controller: 'CallbackController'
                })
                .when('/login/callback/failed', {
                    templateUrl: '/static/components/sdbgui-login/sdbgui-login-failed.template.html',
                    controller: 'FailedController'
                })

            $locationProvider.html5Mode({
                enabled: true,
                requireBase: false
            });
        }
    ]
)

.factory('utils', ['$window', function($window) {
    function base64URL (string) {
            return string.toString(CryptoJS.enc.Base64)
                .replace(/=/g, '')
                .replace(/\+/g, '-')
                .replace(/\//g, '_');
        }

    return {
        generateRandomString: function (length) {
            let text = "";
            let possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-._~";
            for (let i = 0; i < length; i++) {
                text += possible.charAt(Math.floor(Math.random() * possible.length));
            }
            return text;
        },
        generateCodeChallenge: function (code_verifier) {
            return base64URL(CryptoJS.SHA256(code_verifier));
        },

        base64URL: function (string) {
            return base64URL(string);
        },
        goToReferrer: function(ref) {
            ref = (ref) ? ref : $window.sessionStorage.getItem('ssoApp.login_referrer');
            ref = (ref) ? ref : '/';
            window.location = ref;
        },
        hashToObject: function (hash) {
            // Predefine regexs for parsing hash
            let plus2space = /\+/g;
            let paramSplit = /([^&=]+)=?([^&]*)/g;

            // Remove the leading hash
            // let fragment = hash.substring(1);

            let obj = {};

            // Loop until we have no more params
            let param;
            while (true) { // eslint-disable-line no-constant-condition
                param = paramSplit.exec(hash);
                if (!param) { break; }

                let key = param[1];
                let value = param[2];

                // id_token should remain base64url encoded
                if (key === 'id_token' || key === 'access_token' || key === 'code') {
                    obj[key] = value;
                } else {
                    obj[key] = decodeURIComponent(value.replace(plus2space, ' '));
                }
            }
            return obj;
        },
        objToFormUrlencoded:function(obj) {
            // Converts obj to be proper string for Content-Type "application/x-www-form-urlencoded"
            let str = [];
            for (let p in obj)
                str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
            return str.join("&");
        },
        getRedirectUri: function(idp_id) {
            let protocol = window.location.protocol + '//';
            let path = window.location.hostname + '/login/callback';
            let qs = '?idp_id=' + idp_id;
            return protocol + path + qs;
        }
    };
}])

.controller('LoginController', [
    '$scope', '$location', '$httpParamSerializer', '$window', 'slashDB', 'utils',
    function($scope, $location, $httpParamSerializer, $window, slashdb, utils) {
        let ctrl = this;
        ctrl.slashdb = slashdb;
        ctrl.auth_settings = {};
        ctrl.user = '';
        ctrl.password = '';

        function resetLoginErrors() {
            ctrl.loginFormError = '';
            ctrl.loginApiKeyError = '';
            ctrl.loginSSOError = $window.sessionStorage.getItem('ssoApp.login_error');
            $window.sessionStorage.removeItem('ssoApp.login_error');
        }

        function updateController(){
            ctrl.auth_settings = angular.copy(slashdb.settings.auth_settings);
            ctrl.auth_policies = ctrl.auth_settings['authentication_policies']
            let auth_policies = ctrl.auth_policies;
            try {
                ctrl.visibleForm = auth_policies.login_form.enabled && auth_policies.login_form.gui.visible;
            } catch (error) {
                ctrl.visibleForm = false;
            }
            try {
                ctrl.visibleApikey = auth_policies.apikey.enabled && auth_policies.apikey.gui.visible;
            } catch (error) {
                ctrl.visibleApikey = false
            }
            try {
                ctrl.visibleJwt = auth_policies.jwt.enabled && auth_policies.jwt.gui.visible;
                ctrl.idps = auth_policies.jwt.identity_providers;
            } catch (error) {
                ctrl.visibleJwt = false;
                ctrl.idp_id = null;
                ctrl.ssoConfig = null;
            }
        }

        ctrl.login_with_idp = function(idp_id) {
            resetLoginErrors()
            $location.path('/login/sso/' + idp_id);
        };

        ctrl.login_with_form = function () {
            resetLoginErrors();
            slashdb.login(ctrl.user, ctrl.password)
            .then(function (response) {
                utils.goToReferrer();
            })
            .catch(function (response) {
                if (response.status === 429) {
                    ctrl.loginFormError = 'Too many unsuccessful logins. Please check your credentials and try again later.';
                } else {
                    ctrl.loginFormError = response.headers('Warning');
                }
            });
        };

        ctrl.login_with_apikey = function(){
            resetLoginErrors();
            let apikeyConf = ctrl.auth_settings.authentication_policies.apikey;
            let url = '/index.html?' + apikeyConf.apikey_name + "=" + ctrl.apikey;
            slashdb.get(url)
            .then(function(response){
                return slashdb.setupSettings()
            })
            .then(function(response){
                let curr_user = slashdb.settings.user;
                if (curr_user === null || curr_user === 'public'){
                    ctrl.loginApiKeyError = 'Login failed'
                } else {
                    utils.goToReferrer();
                }
            })
        }

        resetLoginErrors();
        if (!ctrl.loginSSOError) {
            // Don't overwrite referrer if handling error
            $window.sessionStorage.setItem('ssoApp.login_referrer', document.referrer);
        }
        slashdb.subscribeSettingsChange($scope, updateController);
        slashdb.setupSettings()
        .then(function(response){
            updateController();
        });
}])

.controller('SSOController', [
    '$scope', '$location', '$httpParamSerializer', '$window', '$routeParams', 'slashDB', 'utils',
    function($scope, $location, $httpParamSerializer, $window, $routeParams, slashdb, utils) {
        let ctrl = this;
        ctrl.slashdb = slashdb;
        ctrl.idp_id = $routeParams.idp_id;
        ctrl.auth_settings = {};

        slashdb.subscribeSettingsChange($scope, updateController);

        function updateController(){
            ctrl.auth_settings = angular.copy(slashdb.settings.auth_settings);
            ctrl.auth_policies = ctrl.auth_settings['authentication_policies']
            let auth_policies = ctrl.auth_policies;
            try {
                ctrl.visibleForm = auth_policies.login_form.enabled && auth_policies.login_form.gui.visible;
            } catch (error) {
                ctrl.visibleForm = false;
            }
            try {
                ctrl.visibleApikey = auth_policies.apikey.enabled && auth_policies.apikey.gui.visible;
            } catch (error) {
                ctrl.visibleApikey = false
            }
            try {
                ctrl.visibleJwt = auth_policies.jwt.enabled && auth_policies.jwt.gui.visible;
                ctrl.idps = auth_policies.jwt.identity_providers;
            } catch (error) {
                ctrl.visibleJwt = false;
                ctrl.idp_id = null;
                ctrl.ssoConfig = null;
            }
        }

        function login() {
            let idp_id = ctrl.idp_id;
            let ssoConfig = ctrl.auth_policies.jwt.identity_providers[idp_id];
            let state = utils.generateRandomString(128);
            let nonce = utils.generateRandomString(128);
            let code_challenge_method = 'S256';
            let code_verifier = utils.generateRandomString(128);
            let code_challenge = utils.generateCodeChallenge(code_verifier);
            $window.sessionStorage.setItem('ssoApp.idp_id', idp_id);
            $window.sessionStorage.setItem('ssoApp.state', state);
            $window.sessionStorage.setItem('ssoApp.nonce', nonce);
            $window.sessionStorage.setItem('ssoApp.code_challenge_method', code_challenge_method);
            $window.sessionStorage.setItem('ssoApp.code_verifier', code_verifier);
            $window.sessionStorage.setItem('ssoApp.code_challenge', code_challenge);

            let qs = {
                'client_id': ssoConfig.client_id,
                'redirect_uri': utils.getRedirectUri(idp_id),
                'response_type': ssoConfig.response_type,
                'response_mode': 'fragment',
                'state': state,
                'nonce': nonce,
                'scope': ssoConfig.scope
            };

            if (ssoConfig.response_type === 'code') {
                qs['code_challenge'] = code_challenge;
                qs['code_challenge_method'] = code_challenge_method;
            }

            qs = $httpParamSerializer(qs);
            window.location = ssoConfig.authorization_endpoint + '?' + qs;
        }

        slashdb.setupSettings()
        .then(function(response){
            updateController();
            login();
        })
}])

.controller('CallbackController', [
    '$scope', '$location', '$window', '$http', 'slashDB', 'utils',
    function($scope, $location, $window, $http, slashdb, utils) {
        let ctrl = this;

        slashdb.subscribeSettingsChange($scope, updateController)

        function updateController() {
            ctrl.auth_settings = angular.copy(slashdb.settings.auth_settings);
            ctrl.auth_policies = ctrl.auth_settings['authentication_policies'];
            ctrl.idps = ctrl.auth_policies.jwt.identity_providers
            ctrl.visibleForm = ctrl.auth_policies.login_form.enabled;
            ctrl.visibleApikey = ctrl.auth_policies.apikey.enabled;
            ctrl.visibleJwt = ctrl.auth_policies.jwt.enabled;
            ctrl.idp_id = $window.sessionStorage.getItem('ssoApp.idp_id');
            ctrl.ssoConfig = ctrl.idps[ctrl.idp_id];
        }

        function authenticate_with_id_token(id_token) {
            /* Use id_token to authenticate in SlashDB. */
            let auth_header = "Bearer " + btoa(id_token);
            return $http.get(
                '/index.html',
                {
                    'headers': {
                        'Authorization': auth_header,
                        'X-Identity-Provider-Id': ctrl.idp_id
                    }
                }
            )
            .then(function(response){
                return slashdb.setupSettings()
            })
            .then(function(response){
                let curr_user = slashdb.settings.user;
                if (curr_user === null || curr_user === 'public'){
                    $window.sessionStorage.setItem('ssoApp.login_error', 'Login failed');
                    utils.goToReferrer('/login');
                } else {
                    utils.goToReferrer();
                }
            })
        }

        function authenticate_with_code(authorization_code) {
            /* Use authorization code from IDP to receive id_token from IDP
            to use that id_token to authenticate in SlashDB. */
            let payload = {
                'client_id': ctrl.ssoConfig.client_id,
                'code_verifier': $window.sessionStorage.getItem('ssoApp.code_verifier'),
                'redirect_uri': utils.getRedirectUri(ctrl.idp_id),
                'grant_type': 'authorization_code',
                'code': authorization_code
            }

            $http({
                method: 'POST',
                url: ctrl.ssoConfig.token_endpoint,
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                transformRequest: utils.objToFormUrlencoded,
                data: payload
            }).then(function (response) {
                authenticate_with_id_token(response.data.id_token)
            });
        }

        function authenticate() {
            let auth_response = angular.extend(
                {},
                utils.hashToObject($location.hash()),
                $location.search()
            )

            if (auth_response.error !== undefined) {
                console.error("Error in authorization response: " + auth_response.error);
                console.error(auth_response.error_description);
                $location.path('/login/callback/failed');
                return null
            }

            let state = $window.sessionStorage.getItem('ssoApp.state');
            if (auth_response.state !== state) {
                console.error('Invalid PKCE state!');
                $location.search('error', 'invalid_pkce_state');
                $location.path('/login/callback/failed');
                return null;
            }

            if (auth_response.code !== undefined) {
                authenticate_with_code(auth_response.code);
            } else if (auth_response.id_token !== undefined) {
                authenticate_with_id_token(auth_response.id_token);
            } else {
                let error_description = "Missing authorization code or id_token in authorization response."
                console.error(error_description)
                $location.search('error', 'missing_auth_code');
                $location.search('error_description', error_description);
                $location.path('/login/callback/failed');
            }
        }

        slashdb.setupSettings()
        .then(function(response){
            updateController();
            authenticate();
        })
}])

.controller('FailedController', [
    '$scope', '$location', '$window', '$http', 'slashDB', 'utils',
    function($scope, $location, $window, $http, slashdb, utils) {
        let ctrl = this;
        let auth_response = angular.extend(
            {},
            utils.hashToObject($location.hash()),
            $location.search()
        )
        $scope.idp_id = auth_response.idp_id;
        $scope.error = auth_response.error;
        $scope.error_description = auth_response.error_description;

}])
;

